<?php
$cwd = realpath(dirname(__FILE__) . '/../../');

class ConfigHelper
{
	public static $defaults;
	private $config = array();

	public function __construct($config = array()) {
		foreach (self::$defaults as $name => $value) {
			if (!isset($config[$name])) {
				trigger_error('The configuration value '.  $name . ' will use the default value ' . $value . '.', E_USER_NOTICE);
				$this->config[$name] = self::$defaults[$name];
			} else {
				$this->config[$name] = $config[$name];
				trigger_error('The configuration value '.  $name . ' has been set to ' . $config[$name] . '.', E_USER_NOTICE);
				unset($config[$name]);
			}
		}

		foreach ($config as $name => $value) {
			trigger_error('The configuration value ' . $name . ' will not be used.', E_USER_WARNING);
		}
	}

	public function load_config($config_file) {
		if (!file_exists($config_file))
		{
			trigger_error('The configuration file ' . $config_file . 'does not exists !', E_USER_ERROR);
		}
		else
		{
			$this->config = array_merge(parse_ini_file($config_file, true), $this->config);
			$lang = explode(',', $this->config['site']['language']);
			$this->config['site']['default_language'] = $lang[0];
		}
	}

	public function get($name) {
		$names = func_get_args();
		$walk = $this->config;
		foreach ($names as $name) {
			$walk = $walk[$name];
		}
		return $walk;
	}

	public function set($array, $value = false) {
		if (is_array($name)) {
			$this->config = array_merge($this->config, $array);
		} else {
			$this->config[$name] = $value;
		}
	}
}

ConfigHelper::$defaults = array(
	'datadir' => realpath($cwd . '/data/'),
	'outputdir' => realpath($cwd . '/public_html/'),
	'tmpdir' => realpath($cwd . '/tmp/'),
);

