<?php
class EntityCategory extends AbstractEntity
{

	protected static $entities = array();
	private $pages = array();
	private $forced = false;

	public static function exists($uid) {
		return isset(self::$entities[$uid]);
	}

	public static function get($uid) {
		return self::$entities[$uid];
	}

	public static function get_all() {
		return self::$entities;
	}

	public function __construct($config) {
		parent::__construct($config);
	}

	public static function get_forced() {
		$entities = self::$entities;
		$forced_list = array();
		foreach ($entities as $uid => $entity) {
			if ($entity->forced) {
				$forced_list[$uid] = $entity;
			}
		}
		return $forced_list;
	}

	public function get_url($lang = null) {
		if ($lang === null) {
			$lang = $this->config->get('site', 'default_language');
		}
		return URLHelper::clean('//' . $this->config->get('site', 'root_url') . '/' . $this->config->get('urls', 'categories') . '/' . $this->uid . '.' . $lang . '.html');
	}

	public static function get_index_url($config, $lang = null) {
		if ($lang === null) {
			$lang = $config->get('site', 'default_language');
		}
		return URLHelper::clean('//' . $config->get('site', 'root_url') . '/' . $config->get('urls', 'categories') . '/' . 'index' . '.' . $lang . '.html');
	}

	public static function generate_index($categories_list, $config, $lang, $smarty) {

		trigger_error('[' . $lang . '] Generating categories list...', E_USER_NOTICE);

		$category = array_shift(array_values(self::$entities));

		if ($category === NULL) {
			trigger_error('[' . $lang . '] Categories index is empty.', E_USER_WARNING);
			return '';
		}

		$langs = explode(',', $config->get('site', 'language'));
		$alt_pages = array();
		foreach ($langs as $alt_lang)
		{
			$alt_pages[] = array(
				'url'       => $category->get_url($alt_lang),
				'lang_code' => strtoupper($alt_lang),
				'icon'      => URLHelper::get_icon_url($alt_lang, 'flags'),
			);
		}

		$smarty->append('page', array(
				'language'    => $lang,
				'title'       => 'Categories',
				'description' => '',
				'keywords'    => '',
				'alt_urls'    => $alt_pages,
				'content'     => array(
					'categories' => $categories_list,
				),
			), true);
		$data = $smarty->fetch('categories.html');
		return $data;
	}

	public function generate($lang, $smarty)
	{
		trigger_error('[' . $lang . '] Generating category ' . $this->uid . '...', E_USER_NOTICE);

		$parser = $this->xml_parser;
		$parser->reset();
		$node = $parser->get_tree('document');

		$langs = explode(',', $this->config->get('site', 'language'));
		$alt_pages = array();
		foreach ($langs as $alt_lang)
		{
			$alt_pages[] = array(
				'url'       => $this->get_url($alt_lang),
				'lang_code' => strtoupper($alt_lang),
				'icon'      => URLHelper::get_icon_url($alt_lang, 'flags'),
			);
		}

		$smarty->append('page', array(
				'language'    => $lang,
				'title'       => $this->get_metadata_value('title', $lang),
				'description' => '',
				'keywords'    => '',
				'alt_urls'    => $alt_pages,
			), true);

		XMLProcessorHelper::recursive_process_document_body($node, $lang, $smarty);

		$pages_list = array();
		foreach ($this->pages as $page)
		{
			$pages_list[] = array(
				'title'  => $page->get_metadata_value('title', $lang),
				'url'    => $page->get_url($lang),
				'legend' => $page->get_metadata_value('description', $lang),
				'icon'   => array(
					'url' => URLHelper::get_icon_url($page->get_metadata_value('icon', $lang)),
				),
			);
		}

		$smarty->append('page', array(
			'content' => array(
				'pages' => $pages_list,
			),
		), true);

		$data = $smarty->fetch('category.html');
		return $data;
	}

	public function add_page($page) {
		if (!in_array($page, $this->pages)) {
			$this->pages[$page->uid] = $page;
		}
		return true;
	}

	public function remove_page($page) {
		if (is_object($page)) {
			if (($key=array_search($this->pages, $page)) !== false) {
				unset($this->pages[$key]);
			}
		} else {
			unset($this->pages[$page]);
		}
		return true;
	}

	public function load($file) {
		$this->xml_parser = XMLProcessorHelper::load($file);
		$this->meta = XMLProcessorHelper::get_file_metadata($this->xml_parser);
		$this->uid = $this->meta['uid'];
		self::$entities[$this->uid] = $this;

		if (!$this->meta['uid'])
		{
			trigger_error('The file named ' . $file . ' does not have a valid uid', E_USER_ERROR);
		}
		elseif (basename($file, '.xml') != $this->meta['uid'])
		{
			trigger_error('The file named ' . $file . ' does not match its document uid ' .  $this->meta['uid'], E_USER_WARNING);
		}
		elseif (isset($this->meta[$this->meta['uid']]))
		{
			trigger_error('Not unique category uid found : ' . $this->meta['uid'], E_USER_ERROR);
		}
		return $this->xml_parser;
	}

	public function set_forced($value) {
		$this->forced = $value;
	}
}
