<?php
class EntityPage extends AbstractEntity
{

	protected static $entities = array();
	private $categories = array();
	private $tags = array();

	public static function exists($uid) {
		return isset(self::$entities[$uid]);
	}

	public static function get($uid) {
		return self::$entities[$uid];
	}

	public static function get_all() {
		return self::$entities;
	}

	public function __construct($config) {
		parent::__construct($config);
	}

	public function get_url($lang = null) {
		if ($lang === null) {
			$lang = $this->config->get('site', 'default_language');
		}
		return URLHelper::clean('//' . $this->config->get('site', 'root_url') . '/' . $this->config->get('urls', 'pages') . '/' . $this->uid . '.' . $lang . '.html');
	}

	public function generate($lang, $smarty)
	{
			trigger_error('[' . $lang . '] Generating page ' . $this->uid . '...', E_USER_NOTICE);

			$parser = $this->xml_parser;
			$parser->reset();
			$node = $parser->get_tree('document');

			$langs = explode(',', $this->config->get('site', 'language'));
			$alt_pages = array();
			foreach ($langs as $alt_lang)
			{
				$alt_pages[] = array(
					'url'       => $this->get_url($alt_lang),
					'lang_code' => strtoupper($alt_lang),
					'icon'      => URLHelper::get_icon_url($alt_lang, 'flags'),
				);
			}

			$smarty->append('page', array(
					'language'    => $lang,
					'title'       => $this->get_metadata_value('title', $lang),
					'description' => $this->get_metadata_value('description', $lang),
					'keywords'    => $this->get_metadata_value('keywords', $lang),
					'icon'        => array(
							     'title' => $this->get_metadata_value('icon', $lang),
							     'url'  => URLHelper::get_icon_url($this->get_metadata_value('icon', $lang)),
							 ),
					'alt_urls'    => $alt_pages,
				), true);


			if ($this->config->get('site', 'generate_tag_pages'))
			{
				$tags = $this->tags;

				$tags_list = array();
				if (!empty($tags))
				{
					foreach ($tags as $uid => $tag)
					{
						$tag = EntityTag::get($uid);
						$tags_list[] = array(
							'url'  => $tag->get_url($lang),
							'title' => $uid,
						);
					}
				}
				$smarty->append('page', array(
						'tags' => $tags_list,
					), true);
			}

			XMLProcessorHelper::recursive_process_document_body($node, $lang, $smarty);
			$data = $smarty->fetch('page.html');
			return $data;
	}

	public function add_category($category) {
		if (!in_array($category, $this->categories)) {
			$this->categories[$category->uid] = $category;
		}
		return true;
	}

	public function remove_category($category) {
		if (is_object($category)) {
			if (($key=array_search($this->categories, $category)) !== false) {
				unset($this->categories[$key]);
			}
		} else {
			unset($this->categories[$category]);
		}
		return true;
	}

	public function add_tag($tag) {
		if (!in_array($tag, $this->tags)) {
			$this->tags[$tag->uid] = $tag;
		}
		return true;
	}

	public function remove_tag($tag) {
		if (is_object($tag)) {
			if (($key=array_search($this->tagstags, $tag)) !== false) {
				unset($this->tags[$key]);
			}
		} else {
			unset($this->tags[$tag]);
		}
		return true;
	}

	public function load($file) {
		$this->xml_parser = XMLProcessorHelper::load($file);
		$this->meta = XMLProcessorHelper::get_file_metadata($this->xml_parser);
		$this->uid = $this->meta['uid'];
		self::$entities[$this->uid] = $this;

		if (!$this->meta['uid'])
		{
			trigger_error('The file named ' . $file . ' does not have a valid uid', E_USER_ERROR);
		}
		elseif (basename($file, '.xml') != $this->meta['uid'])
		{
			trigger_error('The file named ' . $file . ' does not match its document uid ' .  $this->meta['uid'], E_USER_WARNING);
		}
		elseif (isset($this->meta[$this->meta['uid']]))
		{
			trigger_error('Not unique page uid found : ' . $this->meta['uid'], E_USER_ERROR);
		}
		else
		{
			if (isset($this->meta['category']))
			{
				$this->add_category(EntityCategory::get($this->meta['category']));
				EntityCategory::get($this->meta['category'])->add_page($this);
			}
			if (isset($this->meta['tags']))
			{
				foreach	($this->meta['tags'] as $lang => $tags)
				{
					$tags = explode(',', $tags);
					foreach ($tags as $tag_uid)
					{
						$tag = new EntityTag($tag_uid, $this->config);
						$tag = EntityTag::get($tag_uid);
						$this->add_tag($tag);
						$tag->add_page($this);
					}
					$this->meta['tags'][$lang] = $tags;
				}
			}
		}
		return $this->xml_parser;
	}
}

