<?php
class EntityTag extends AbstractEntity
{

	protected static $entities = array();
	private $pages = array();
	private $forced = false;

	public static function exists($uid) {
		return isset(self::$entities[$uid]);
	}

	public static function get($uid) {
		return self::$entities[$uid];
	}

	public static function get_all() {
		return self::$entities;
	}

	public function __construct($uid, $config) {
		if (!self::exists($uid)) {
			parent::__construct($config);
			self::$entities[$uid] = $this;
			$this->uid = $uid;
		}
	}

	public static function get_forced() {
		$entities = self::$entities;
		$forced_list = array();
		foreach ($entities as $uid => $entity) {
			if ($entity->forced) {
				$forced_list[$uid] = $entity;
			}
		}
		return $forced_list;
	}

	public function get_url($lang = null) {
		if ($lang === null) {
			$lang = $this->config->get('site', 'default_language');
		}
		return URLHelper::clean('//' . $this->config->get('site', 'root_url') . '/' . $this->config->get('urls', 'tags') . '/' . $this->uid . '.' . $lang . '.html');
	}

	public static function get_index_url($config, $lang = null) {
		if ($lang === null) {
			$lang = $config->get('site', 'default_language');
		}
		return URLHelper::clean('//' . $config->get('site', 'root_url') . '/' . $config->get('urls', 'tags') . '/' . 'index' . '.' . $lang . '.html');
	}

	public static function generate_index($tags_list, $config, $lang, $smarty) {

		trigger_error('[' . $lang . '] Generating tags list...', E_USER_NOTICE);

		$tag = array_shift(array_values(self::$entities));

		if ($tag === NULL) {
			trigger_error('[' . $lang . '] Tags index is empty.', E_USER_WARNING);
			return '';
		}

		$langs = explode(',', $config->get('site', 'language'));
		$alt_pages = array();
		foreach ($langs as $alt_lang)
		{
			$alt_pages[] = array(
				'url'       => $tag->get_url($alt_lang),
				'lang_code' => strtoupper($alt_lang),
				'icon'      => URLHelper::get_icon_url($alt_lang, 'flags'),
			);
		}

		$smarty->append('page', array(
				'language'    => $lang,
				'title'       => 'Tags',
				'description' => '',
				'keywords'    => '',
				'alt_urls'    => $alt_pages,
				'content'     => array(
					'tags' => $tags_list,
				),
			), true);
		$data = $smarty->fetch('tags.html');
		return $data;
	}

	public function generate($lang, $smarty)
	{
		trigger_error('[' . $lang . '] Generating tag ' . $this->uid . '...', E_USER_NOTICE);

		$smarty->append('page', array(
				'language'    => $lang,
				'title'       => $this->uid,
				'description' => '',
				'keywords'    => '',
			), true);

		$pages_list = array();
		foreach ($this->pages as $page)
		{
			$pages_list[] = array(
				'title'  => $page->get_metadata_value('title', $lang),
				'url'    => $page->get_url($lang),
				'legend' => $page->get_metadata_value('description', $lang),
				'icon'   => array(
					'url' => URLHelper::get_icon_url($page->get_metadata_value('icon', $lang)),
				),
			);
		}

		$smarty->append('page', array(
			'content' => array(
				'pages' => $pages_list,
			),
		), true);

		$data = $smarty->fetch('tag.html');
		return $data;
	}

	public function add_page($page) {
		if (!in_array($page, $this->pages)) {
			$this->pages[$page->uid] = $page;
		}
		return true;
	}

	public function remove_page($page) {
		if (is_object($page)) {
			if (($key=array_search($this->pages, $page)) !== false) {
				unset($this->pages[$key]);
			}
		} else {
			unset($this->pages[$page]);
		}
		return true;
	}

	public function load($file) { }

	public function set_forced($value) {
		$this->forced = $value;
	}
}
