<?php
$cwd = realpath(dirname(__FILE__));
include_once($cwd . '/../contrib/smarty/libs/Smarty.class.php');

//error_reporting(E_ALL ^ E_USER_NOTICE);

set_error_handler('error_handler', E_USER_NOTICE | E_USER_WARNING | E_USER_ERROR);

function error_handler($errno, $errstr, $errfil, $errline)
{
	if (error_reporting() & $errno)
	{
		switch ($errno)
		{
			case E_USER_NOTICE:
				echo '[Notice] ' . $errstr;
				echo "\n";
			break;

			case E_USER_WARNING:
				echo '[Warning] ' . $errstr;
				echo "\n";
			break;

			case E_USER_ERROR:
				echo '[Error] ' . $errstr;
				echo "\n";
				die();
			break;

			default:
				return false;
			break;
		}
	}
}

class Liseron
{
	private $config = array();
	private $meta = array();
	private $tags = array();
	private $categories = array();
	private $sitemap = array();
	private $force_generate = array();
	private $error_level = E_ALL;
	private $xml_files = array();


	/**
	 */
	public function __construct( $args )
	{
		$this->config = new ConfigHelper($args);
		$this->config->load_config($this->config->get('datadir') . '/site.ini');
		XMLProcessorHelper::init($this->config);
		URLHelper::init($this->config);
	}


	/**
	 */
	public function load_xml_files()
	{
		$files_list = scandir($this->config->get('datadir') . '/' . $this->config->get('directories', 'xml_templates') . '/categories/');
		foreach ($files_list as $file)
		{
			if (strtoupper(substr($file, -3)) == 'XML')
			{
				$category = new EntityCategory($this->config);
				$category->load($this->config->get('datadir') . '/' . $this->config->get('directories', 'xml_templates') . '/categories/' . $file);
			}
		}

		$files_list = scandir($this->config->get('datadir') . '/' . $this->config->get('directories', 'xml_templates') . '/pages/');
		foreach ($files_list as $file)
		{
			if (strtoupper(substr($file, -3)) == 'XML')
			{
				$page = new EntityPage($this->config);
				$page->load($this->config->get('datadir') . '/' . $this->config->get('directories', 'xml_templates') . '/pages/' . $file);
			}
		}
	}


	/**
	 */
	private function smarty_assign_globals($smarty)
	{
		$smarty->append('site', array(
				'language'    => $this->config->get('site', 'default_language'),
				'title'       => $this->config->get('site', 'title'),
				'subtitle'    => $this->config->get('site', 'subtitle'),
				'author'      => $this->config->get('site', 'author'),
				'keywords'    => $this->config->get('site', 'keywords'),
				'description' => $this->config->get('site', 'description'),
				'copyright'   => $this->config->get('site', 'copyright'),
			), true);
	}


	/**
	 */
	private function write_html_file($type, $uid, $lang, $data)
	{
		switch ($type)
		{
			case 'page':
				$key = 'pages';
			break;

			case 'category':
				$key = 'categories';
			break;

			case 'tag':
				$key = 'tags';
			break;

			default:
				trigger_error('Invalid file type : ' . $type, E_USER_ERROR);
				return;
			break;
		}
		if (!file_exists($this->config->get('outputdir') . '/' . $this->config->get('directories', $key))) {
			mkdir($this->config->get('outputdir') . '/' . $this->config->get('directories', $key), 0777, true);
		}
		$handle = fopen($this->config->get('outputdir') . '/' . $this->config->get('directories', $key) . '/' . $uid . '.' . $lang . '.html', 'w');
		if ($handle)
		{
			fwrite($handle, $data);
		}
		fclose($handle);
	}

	/**
	 */
	private function write_sitemap($data)
	{
		if (!file_exists($this->config->get('outputdir') . '/' . $this->config->get('directories', 'sitemap'))) {
			mkdir($this->config->get('outputdir') . '/' . $this->config->get('directories', 'sitemap'), 0777, true);
		}
		$handle = fopen($this->config->get('outputdir') . '/' . $this->config->get('directories', 'sitemap') . '/sitemap.xml', 'w');
		if ($handle)
		{
			fwrite($handle, $data);
		}
		fclose($handle);
	}


	/**
	 */
	private function generate_pages($lang)
	{
		$pages = EntityPage::get_all();
		foreach ($pages as $uid => $page)
		{
			$smarty = new SmartyInterface($this->config);
			$this->smarty_assign_globals($smarty);
			$data = $page->generate($lang, $smarty);
			$this->write_html_file('page', $uid, $lang, $data);

			$this->sitemap[] = array(
				'url' => $page->get_url(),
			);
		}
	}


	/**
	 */
	private function generate_categories($lang)
	{
		$categories_list = array();
		if ($this->config->get('site', 'generate_category_pages'))
		{
			$categories = EntityCategory::get_all();
			foreach ($categories as $uid => $category)
			{
				$categories_list[] = array(
					'url'  => $category->get_url($lang),
					'title' => $category->get_metadata_value('title', $lang),
				);

				$smarty = new SmartyInterface($this->config);
				$this->smarty_assign_globals($smarty);
				$data = $category->generate($lang, $smarty);
				$this->write_html_file('category', $uid, $lang, $data);

				$this->sitemap[] = array(
					'url' => $category->get_url($lang)
				);
			}
		}
		else
		{
			$categories = EntityCategory::get_forced();
			foreach ($categories as $uid => $category)
			{
				$categories_list[] = array(
					'url'  => $category->get_url($lang),
					'title' => $category->get_metadata_value('title', $lang),
				);

				$smarty = new SmartyInterface($this->config);
				$this->smarty_assign_globals($smarty);
				$data = $category->generate($lang, $smarty);
				$this->write_html_file('category', $uid, $lang, $data);

				$this->sitemap[] = array(
					'url' => $category->get_url($lang),
				);
			}
		}

		$smarty = new SmartyInterface($this->config);
		$this->smarty_assign_globals($smarty);

		$data = EntityCategory::generate_index($categories_list, $this->config, $lang, $smarty);

		$this->write_html_file('category', 'index', $lang, $data);

		$this->sitemap[] = array(
			'url' => EntityCategory::get_index_url($this->config, $lang),
		);
	}


	/**
	 */
	private function generate_tags($lang)
	{
		$tags_list = array();
		if ($this->config->get('site', 'generate_tag_pages'))
		{
			$tags = EntityTag::get_all();
			foreach ($tags as $uid => $tag)
			{
				$tags_list[] = array(
					'url'  => $tag->get_url($lang),
					'title' => $uid,
				);

				$smarty = new SmartyInterface($this->config);
				$this->smarty_assign_globals($smarty);
				$data = $tag->generate($lang, $smarty);
				$this->write_html_file('tag', $uid, $lang, $data);

				$this->sitemap[] = array(
					'url' => $tag->get_url(),
				);
			}
		}
		else
		{
			$tags = EntityTag::get_forced();
			foreach ($tags as $uid => $tag)
			{
				$tags_list[] = array(
					'url'  => $tag->get_url($lang),
					'title' => $uid,
				);

				$smarty = new SmartyInterface($this->config);
				$this->smarty_assign_globals($smarty);
				$data = $tag->generate($lang, $smarty);
				$this->write_html_file('tag', $uid, $lang, $data);

				$this->sitemap[] = array(
					'url' => $tag->get_url(),
				);
			}
		}

		$smarty = new SmartyInterface($this->config);
		$this->smarty_assign_globals($smarty);

		$data = EntityTag::generate_index($tags_list, $this->config, $lang, $smarty);

		$this->write_html_file('tag', 'index', $lang, $data);

		$this->sitemap[] = array(
			'url' => EntityTag::get_index_url($this->config, $lang)
		);
	}


	/**
	 */
	private function generate_sitemap()
	{
		$langs = explode(',', $this->config->get('site', 'language'));

		$data = '<?xml version="1.0" encoding="UTF-8" ' . '?' . '><urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">'; // Concatenation of '?' is a trick to prevent breaking syntax highlighting in EDI

		foreach ($langs as $lang)
		{
			trigger_error('[' . $lang . '] Generating sitemap...', E_USER_NOTICE);

			foreach ($this->sitemap as $entry)
			{
				$data .= '<url><loc>' . $entry['url'] . '</loc></url>';
			}

		}

		$data .= '</urlset>';

		$this->write_sitemap($data);
	}


	/**
	 */
	public function generate()
	{
		$this->load_xml_files();
		$langs = explode(',', $this->config->get('site', 'language'));
		foreach ($langs as $lang) {
			$this->generate_pages($lang);
			$this->generate_categories($lang);
			$this->generate_tags($lang);
		}
		$this->generate_sitemap();
	}
}

?>
