<?php

class XMLParser
{
	private $parser;
	private $opened_file;

	public function __construct()
	{
		$this->parser = new XMLReader();
	}

	public function get_parser()
	{
		return $this->parser;
	}

	public function init($file)
	{
		$this->opened_file = $file;
		$this->parser->open($file);
		$this->parser->setParserProperty(XMLReader::VALIDATE, true);
	}

	public function reset()
	{
		$this->parser->close();
		$this->parser->open($this->opened_file);
	}

	public function is_valid()
	{
		return $this->parser->isValid();
	}

	private function read_node()
	{
		$node['@name'] = $this->parser->name;
		$node['@value'] = $this->parser->value;
		$node['@type'] = $this->parser->nodeType;
		$node['@attr'] = array();

		if ($this->parser->hasAttributes)
		{
			$nattr = $this->parser->attributeCount;
			for ($i=0; $i < $nattr; $i++)
			{
				$this->parser->moveToNextAttribute();
				$node['@attr'][$this->parser->name] = $this->parser->value;
			}
		}
		return $node;
	}

	public function get_node($node_name)
	{
		while ($this->parser->read() && ($this->parser->nodeType != XMLReader::ELEMENT || $this->parser->name != $node_name))
		{
			continue;
		}
		$node = $this->read_node();
		return $node;
	}

	public function get_tree($node_name)
	{
		while ($this->parser->read() && ($this->parser->nodeType != XMLReader::ELEMENT || $this->parser->name != $node_name))
		{
			continue;
		}

		$stack = array();
		$node = array();
		$node['@name'] = $this->parser->name;
		$node['@type'] = $this->parser->nodeType;
		if ($this->parser->hasAttributes)
		{
			$nattr = $this->parser->attributeCount;
			for ($i=0; $i < $nattr; $i++)
			{
				$this->parser->moveToNextAttribute();
				$node['@attr'][$this->parser->name] = $this->parser->value;
			}
		}
		array_push($stack, $node);
		while ($this->parser->read() && ($this->parser->nodeType != XMLReader::END_ELEMENT || $this->parser->name != $node_name))
		{
			$buff = array();
			switch ($this->parser->nodeType)
			{
				case XMLReader::ELEMENT:
					$buff['@name'] = $this->parser->name;
					$buff['@type'] = XMLReader::ELEMENT;
					$isEmpty = $this->parser->isEmptyElement; // Context-base property, must be stored before moving to attributes
					if ($this->parser->hasAttributes)
					{
						$nattr = $this->parser->attributeCount;
						for ($i=0; $i < $nattr; $i++)
						{
							$this->parser->moveToNextAttribute();
							$buff['@attr'][$this->parser->name] = $this->parser->value;
						}
					}
					if (!$isEmpty)
					{
						array_push($stack, $buff);
					}
					else
					{
						$node = array_pop($stack);
						$node['@child'][] = $buff;
						array_push($stack, $node);
					}
				break;

				case XMLReader::TEXT:
				case XMLReader::CDATA:
					$buff['@type'] = $this->parser->nodeType;
					$buff['@value'] = $this->parser->value;
					$node = array_pop($stack);
					$node['@child'][] = $buff;
					array_push($stack, $node);
				break;

				case XMLReader::END_ELEMENT:
					$buff = array_pop($stack);
					$node = array_pop($stack);
					$node['@child'][] = $buff;
					array_push($stack, $node);
					/*
					switch ($buff['@type'])
					{
						case XMLReader::ELEMENT:
							$node = array_pop($stack);
							$node[$buff['@name']] = $buff;
							unset($node[$buff['@name']]['@name']);
							array_push($stack, $node);
						break;

						case XMLReader::TEXT:
						case XMLReader::CDATA:
							$node = array_pop($stack);
							$node[$buff['@name']] = $buff;
							unset($node[$buff['@name']]['@name']);
							array_push($stack, $node);
						break;

						default:
						break;
					}
					*/
				break;

				default:
				break;
			}
			continue;
		}

		return $stack[0];
	}

}



?>
