<?php

class XMLProcessorHelper {


	private static $config;
	private static $bbcode_parser;

	public static function init($config) {
		self::$config = $config;
		self::$bbcode_parser = new BBCodeInterface();
	}

	public static function load($path) {
		$parser = new XMLParser();
		$parser->init($path);
		if ($parser->is_valid())
		{
			trigger_error('The file named ' . basename($path) . ' will be parsed', E_USER_NOTICE);
			$node = $parser->get_node('LiseronTemplate');
			if ($node['@attr']['version'] > 1.0)
			{
				trigger_error('Unsupported format version for ' . basename($path) . ': ' . $node['@attr'][0]['version'], E_USER_ERROR);
			}
		}
		else
		{
			trigger_error('The document named ' . basename($path) . ' is not a valid XML file', E_USER_ERROR);
		}
		return $parser;

	}

	public static function load_category($uid) {
		return self::load(self::$config->get('directories', 'xml_templates') . '/categories/' . $uid . '.xml');
	}

	public static function load_page($uid)
	{
		return self::load(self::$config->get('directories', 'xml_templates') . '/pages/' . $uid . '.xml');
	}

	public static function get_file_metadata($parser)
	{
		$tree = $parser->get_tree('meta');
		$meta = array();
		$tree = $tree['@child'];
		foreach ($tree as $node)
		{
			if ($node['@type'] == XMLReader::ELEMENT && ($node['@child'][0]['@type'] == XMLReader::TEXT || $node['@child'][0]['@type'] == XMLReader::CDATA))
			{
				if (isset($node['@attr']['lang']))
				{
					$langs = explode(',', $node['@attr']['lang']);
					foreach ($langs as $lang)
					{
						$meta[$node['@name']][$lang] = $node['@child'][0]['@value'];
					}
				}
				else
				{
					$meta[$node['@name']] = $node['@child'][0]['@value'];
				}
			}
			else
			{
				trigger_error('Error while parsing meta data : node ' . $node['@name'] . ' is not a valid XML element', E_USER_ERROR);
			}
		}
		return $meta;
	}


	private static function recursive_process_inner_children($node, $lang, $context = array())
	{
		static $stack_level = -1;
		$smarty = new SmartyInterface(self::$config);
		$data = '';

		$stack_level++;
		if (isset($node['@type']))
		{
			switch ($node['@type'])
			{
				case XMLReader::ELEMENT:
					if (!@$node['@attr']['lang'] || $node['@attr']['lang'] == $lang)
					{
						switch ($node['@name'])
						{
							case 'list':
								$smarty->assign('title', @$node['@attr']['title']);
								if (@$node['@attr']['type'] == 'links')
								{
									$context = array('list_type' => 'links');
								}
								else
								{
									$context = array();
								}
								foreach ($node['@child'] as $child)
								{
									$html = self::recursive_process_inner_children($child, $lang, $context);
									$smarty->append('content', $html);
								}
								if (@$context['list_type'] == 'links')
								{
									$data .= $smarty->fetch('_list_link.html');
								}
								else
								{
									$data .= $smarty->fetch('_list.html');
								}
								$context = array();
							break;

							case 'link':
								if (@$node['@attr']['for'])
								{
									$prefix = substr($node['@attr']['for'], 0, strpos($node['@attr']['for'], ':'));
									if ($prefix == 'tag' || $prefix == 'category')
									{
										$name = substr($node['@attr']['for'], strpos($node['@attr']['for'], ':') + 1);
										if ($prefix == 'category') {
											EntityCategory::get($name)->set_forced(true);
										}
										if ($prefix == 'tag') {
											EntityTag::get($name)->set_forced(true);
										}
										$smarty->assign('link', array(
											'url'    => URLHelper::get_entity_url($prefix, $name, $lang),
											'title'  => (@$node['@attr']['name'] ? $node['@attr']['name'] : $name),
											'legend' => @$node['@attr']['legend'],
										));
										if (@$node['@attr']['icon'])
										{
											$smarty->assign('icon', array(
												'url' => URLHelper::get_icon_url($node['@attr']['icon']),
											));
										}
										else
										{
											$smarty->assign('icon', array(
												'url' => URLHelper::get_icon_url('tag'),
											));
										}
									}
									else
									{
										$uid = $node['@attr']['for'];
										$smarty->assign('link', array(
											'url'    => URLHelper::get_entity_url('page', $uid, $lang),
											'title'  => Entitypage::get($uid)->get_metadata_value('title', $lang),
											'legend' => ( @$node['@attr']['legend'] ?
												$node['@attr']['legend'] : Entitypage::get($uid)->get_metadata_value('description', $lang) ),
										));

										if (@$node['@attr']['icon'])
										{
											$smarty->assign('icon', array(
												'url' => URLHelper::get_icon_url($node['@attr']['icon']),
											));
										}
										elseif (EntityPage::get($node['@attr']['for'])->get_metadata_value('icon', $lang))
										{
											$smarty->assign('icon', array(
												'url' => URLHelper::get_icon_url(EntityPage::get($uid)->get_metadata_value('icon', $lang)),
											));
										}
									}
								}
								else
								{
									$smarty->assign('link', array(
										'url'    => URLHelper::get_external_url($node['@attr']['url']),
										'title'  => $node['@attr']['name'],
										'legend' => @$node['@attr']['legend'],
									));
									if (@$node['@attr']['icon'])
									{
										$smarty->assign('icon', array(
											'url' => URLHelper::get_icon_url($node['@attr']['icon']),
										));
									}
									else
									{
										$smarty->assign('icon', array(
											'url' => URLHelper::get_icon_url('external', 16),
										));
									}
								}
								if (@$context['list_type'] == 'links')
								{
									$data .= $smarty->fetch('_list_link_item.html');
								}
								else
								{
									$data .= $smarty->fetch('_link.html');
								}
							break;

							case 'text':
								$data .= self::recursive_process_inner_children($node['@child'], $lang);
							break;

							case 'em':
							case 'b':
							case 'i':
							case 's':
							case 'u':
								$data .= '<' . $node['@name'] . '>' . self::recursive_process_inner_children($node['@child'], $lang) . '</' . $node['@name'] . '>';
							break;

							default:
							break;
						}
					}
				break;

				case XMLReader::TEXT:
					//$data .= nl2br(trim($node['@value']));
					$data .= nl2br(self::$bbcode_parser->qparse(trim($node['@value'])));

					if ($stack_level == 1)
					{
						$data = '<p>' . $data . '</p>';
					}
				break;

				case XMLReader::CDATA:
					$data = trim($node['@value']);
				break;

				default:
				break;
			}
		}
		else
		{
			foreach ($node as $child)
			{
				$data .= self::recursive_process_inner_children($child, $lang);
			}
		}
		$stack_level--;
		return $data;
	}


	public static function recursive_process_document_body($node, $lang, $smarty)
	{
		switch ($node['@type'])
		{
			case XMLReader::ELEMENT:
				if (!@$node['@attr']['lang'] || $node['@attr']['lang'] == $lang)
				{
					/*
					if (isset($node['@child']))
					{
						foreach ($node['@child'] as $element)
						{
							$data = self::recursive_process_document_body($element, $lang, $smarty);
						}
					}
					 */

					switch ($node['@name'])
					{
						case 'document':
							foreach ($node['@child'] as $child)
							{
								self::recursive_process_document_body($child, $lang, $smarty);
							}
						break;

						case 'intro':
							$smarty->append('page', array('intro' => array('title' => @$node['@attr']['title'])), true);
							$smarty->append('page', array('intro' => array('html' => self::recursive_process_inner_children($node['@child'], $lang))), true);
						break;

						case 'content':
							$smarty->append('page', array('content' => array('title' => @$node['@attr']['title'])), true);
							$smarty->append('page', array('content' => array('html' => self::recursive_process_inner_children($node['@child'], $lang))), true);
						break;

						default:
						break;
					}
				}
			break;

			case XMLReader::TEXT:
			case XMLReader::CDATA:
				//return $node['@value'];
				//return self::bbcode_parser->qparse($node['@value']);
			break;

			default:
			break;
		}
	}
}
